/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <cassert>
#include <cmath>

#include <vector>
#include <algorithm>
#include <fstream>
#include <iostream>

#define C10 2.2E+07
#define C20 100.
#define C30 30.

#define EMIN 0.1 // MeV
#define EMAX 2470. // MeV

#define NOF_BINS_LIN 100
#define NOF_BINS_LOG 100

namespace {

std::vector<double>
equal_distant_points( double lower, double upper, unsigned int points)
{
	assert(points > 1);
	assert(upper > lower);

	std::vector<double> data(points + 1);

	double step = (upper - lower) / points;
	double value = lower;
	for ( unsigned int i = 0; i < points + 1; ++i, value += step)
		data[i] = value;

	return data;
}

std::vector<double>
log_distant_points( double lower, double upper, unsigned int points)
{
	assert(points > 1);
	assert(upper > lower);

	std::vector<double> data(points + 1);

	double step = log(upper/lower) / points;
	double value = lower;
	for ( unsigned int i = 0; i < points + 1; ++i, value *= exp(step))
		data[i] = value;

	return data;
}

double
exp_fluence_diff(double energy)
{
	return (C10 * exp(-(energy - C20) / C30));
}

double
exp_fluence_integral(double energy)
{
	return (- C10 * C30 * exp(-(energy - C20) / C30));
}

} // namespace

int
main( int argc, char** argv)
{
	// linear and logarithmic scale of energy
	std::vector<double> enelin, enelog;
	std::vector<double> flulin, flulog;
	enelin = equal_distant_points( EMIN, EMAX, NOF_BINS_LIN);
	enelog = log_distant_points( EMIN, EMAX, NOF_BINS_LOG);

	flulin.resize(enelin.size());
	flulog.resize(enelog.size());
	std::transform( enelin.begin(), enelin.end(), flulin.begin(), exp_fluence_diff);
	std::transform( enelog.begin(), enelog.end(), flulog.begin(), exp_fluence_diff);

	std::ofstream testlin, testlog;
	testlin.open( "flulin.dat", std::ofstream::out);
	testlog.open( "flulog.dat", std::ofstream::out);

	testlin << std::scientific;
	testlog << std::scientific;
	for ( size_t i = 0; i < enelin.size(); ++i) {
		testlin << enelin[i] << " " << flulin[i] << std::endl;
	}
	for ( size_t i = 0; i < enelog.size(); ++i) {
		testlog << enelog[i] << " " << flulog[i] << std::endl;
	}
	
	std::cout << "Differential fluence dF/dE = C10 * exp(-(E-C20)/C30)" << std::endl;
	std::cout << "Integral fluence [protons/cm^2]: " << exp_fluence_integral(EMAX)-exp_fluence_integral(EMIN) << std::endl;

	testlin.close();
	testlog.close();
 
	return 0;
}
